﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Magic.Utils;

namespace AbilityParser.Ability
{
	class AbilityAlternatives : IAbility
	{
		internal bool CanChooseOne;
		internal bool CanChooseTwo;
		internal bool IsOpponentChoosing;

		internal List<IAbility> Options;

		public bool IsParsed()
		{
			return Options.All( o => o.IsParsed() );
		}

		public IEnumerable<UnknownPart> GetUnknownParts()
		{
			return Options.SelectMany( o => o.GetUnknownParts() );
		}

		public bool IsProxy()
		{
			return Options.Any( o => IsProxy() );
		}

		public IAbility RemoveProxy()
		{
			for( int i = 0; i < Options.Count; i++ )
			{
				Options[i] = Options[i].RemoveProxy();
			}
			return this;
		}

		public override string ToString()
		{
			StringBuilder sbText = new StringBuilder();
			sbText.Append( IsOpponentChoosing ? "Opp chooses " : "You choose ");
			if (CanChooseOne) sbText.Append("1");
			if (CanChooseOne && CanChooseTwo) sbText.Append(" or ");
			if (CanChooseTwo) sbText.Append("2");
			sbText.Append( ":" );
			foreach( IAbility ability in Options )
			{
				sbText.AppendFormat("{0} * {1}", Environment.NewLine, ability);
			}

			return sbText.ToString();
		}

		public AbContext SubParts { get; set; }

		public static readonly Parser<IAbility> Parser = new AlternativesParser();


		public class AlternativesParser : Parser<IAbility>
		{
			public const string Choose = "choose";
			public static readonly string[] unions = new[] {"; or ", "; and/or ", ";"};	 // Phyrexian splicer has ';' - others have union

			public override bool Can( string text )
			{
				return text.IndexOf( CommonTokens.EmDash ) >= 0 && text.IndexOf( Choose, StringComparison.OrdinalIgnoreCase ) >= 0;
			}

			protected override IAbility Parse( string text )
			{
				AbilityAlternatives result = new AbilityAlternatives();
				int dashPosition = text.IndexOf( CommonTokens.EmDash );
				string choosePart = text.Substring( 0, dashPosition );
				result.CanChooseOne = choosePart.IndexOf( "one", StringComparison.OrdinalIgnoreCase ) >= 0;
				result.CanChooseTwo = choosePart.IndexOf( "two", StringComparison.OrdinalIgnoreCase) >= 0
					|| choosePart.IndexOf( "both", StringComparison.OrdinalIgnoreCase) >= 0;
				result.IsOpponentChoosing = choosePart.IndexOf( CommonTokens.Opponent ) >= 0;

				string choiceList = text.Substring( dashPosition + CommonTokens.EmDashSpace.Length );
				string[] choices = choiceList.Split( unions, StringSplitOptions.RemoveEmptyEntries );
				result.Options = choices.Select( c => new UnparsedAbility( c.Trim() ) ).Cast<IAbility>().ToList();

				return result;
			}
		}

	}


}
