﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Magic.Utils;

namespace AbilityParser.Ability
{
	class AbilitySequence : IAbility
	{
		internal List<IAbility> Actions;

		public bool IsParsed()
		{
			return Actions.All( o => o.IsParsed() );
		}

		public IEnumerable<UnknownPart> GetUnknownParts()
		{
			return Actions.SelectMany( o => o.GetUnknownParts() );
		}

		public bool IsProxy()
		{
			return Actions.Any( o => IsProxy() );
		}

		public IAbility RemoveProxy()
		{
			for( int i = 0; i < Actions.Count; i++ )
			{
				Actions[i] = Actions[i].RemoveProxy();
			}
			return this;
		}

		public override string ToString()
		{
			StringBuilder sbText = new StringBuilder();
			foreach( IAbility ability in Actions )
			{
				sbText.AppendLine(ability.ToString());
			}

			return sbText.ToString();
		}

		public AbContext SubParts { get; set; }

		public static readonly Parser<IAbility> Parser = new AlternativesParser();


		public class AlternativesParser : Parser<IAbility>
		{
			public override bool Can( string text )
			{
				return text.IndexOf( CommonTokens.PeriodSpace[0] ) >= 0;
			}

			protected override IAbility Parse( string text )
			{
				AbilitySequence result = new AbilitySequence();
				string[] choices = text.Split( CommonTokens.PeriodSpace, StringSplitOptions.RemoveEmptyEntries );
				result.Actions = choices.Select( c => new UnparsedAbility( c ) ).Cast<IAbility>().ToList();
				return result;
			}
		}

	}


}
