﻿using System;
using System.Collections.Generic;
using AbilityParser.Condition;
using AbilityParser.MagicPredicate;
using Magic.Utils;

namespace AbilityParser.Ability
{
	class ConditionedAbility : IAbility
	{
		internal ICondition Condition;
		internal IAbility Ability;
		internal bool Continuos;

		public override string ToString()
		{
			string condition = Condition.ToString();
			string effect = Ability.ToString();
			return string.Format( Continuos ? "while({0}) {{ {1} }}" : "if ({0}) {{ {1} }}", condition, effect);
		}

		public bool IsParsed()
		{
			return Condition.IsParsed() && Ability.IsParsed();
		}

		public IEnumerable<UnknownPart> GetUnknownParts()
		{
			List<UnknownPart> unknowns = new List<UnknownPart>();
			unknowns.AddRange(Condition.GetUnknownParts());
			unknowns.AddRange(Ability.GetUnknownParts());
			return unknowns;
		}

		public bool IsProxy()
		{
			return Condition.IsProxy() || Ability.IsProxy();
		}

		public IAbility RemoveProxy()
		{
			Condition = Condition.RemoveProxy();
			Ability = Ability.RemoveProxy();
			return this;
		}

		public AbContext SubParts
		{
			get { return Ability.SubParts; }
			set { Ability.SubParts = value; }
		}

		public static Parser<IAbility> ParseUntil = new UntilParser();
		public static Parser<IAbility> ParseIf = new IfParser();
		public static Parser<IAbility> ParseAsLongAs = new AlaParser();

		class UntilParser : Parser<IAbility>
		{
			public override bool Can( string text )
			{
				return text.IndexOf(CommonTokens.Until, StringComparison.OrdinalIgnoreCase) == 0 && text.IndexOf( CommonTokens.CommaSpace ) > 0;
			}

			protected override IAbility Parse( string text )
			{
				ConditionedAbility result = new ConditionedAbility();
				int iCs = text.IndexOf( CommonTokens.CommaSpace );
				string abilityText = text.Substring( iCs + CommonTokens.CommaSpace.Length );
				string timeLimit = text.Substring( CommonTokens.Until.Length, iCs - CommonTokens.Until.Length );
				result.Ability = new UnparsedAbility( abilityText );
				result.Condition = new UntilTime { MagicPredicate = new UnparsedTurnPart(timeLimit)};
				return result;
			}
		}

		class IfParser : Parser<IAbility>
		{
			public override bool Can( string text )
			{
				return text.StartsWith( CommonTokens.If, StringComparison.OrdinalIgnoreCase ) && text.IndexOf( CommonTokens.CommaSpace ) > 0;
			}

			protected override IAbility Parse( string text )
			{
				ConditionedAbility result = new ConditionedAbility();
				int iCs = IndexOfCommaSpaceAfterSiblings(text);
				result.Ability = new UnparsedAbility( text.Substring(iCs + CommonTokens.CommaSpace.Length) );
				result.Condition = new UnparsedCondition( text.Substring(CommonTokens.If.Length, iCs - CommonTokens.If.Length) );
				return result;
			}
		}

		class AlaParser : Parser<IAbility>
		{
			public override bool Can(string text)
			{
				return text.StartsWith(CommonTokens.AsLongAs, StringComparison.OrdinalIgnoreCase) && text.IndexOf(CommonTokens.CommaSpace) > 0;
			}

			protected override IAbility Parse(string text)
			{
				ConditionedAbility result = new ConditionedAbility();
				int iCs = IndexOfCommaSpaceAfterSiblings(text);
				result.Ability = new UnparsedAbility(text.Substring(iCs + CommonTokens.CommaSpace.Length));
				result.Condition = new UnparsedCondition(text.Substring(CommonTokens.AsLongAs.Length, iCs - CommonTokens.AsLongAs.Length));
				result.Continuos = true;
				return result;
			}
		}
	}
}
