﻿using System;
using System.Collections.Generic;
using System.Linq;
using AbilityParser.Trigger;
using Magic.Utils;

namespace AbilityParser.Ability
{
	class ReplacementAbility : IAbility
	{
		internal ITrigger Replaceable;
		internal IAbility Ability;
		internal bool IsOneShot;

		public override string ToString()
		{
			return string.Format("{2} [{0}],\n instead {{ {1} }}", Replaceable, Ability, IsOneShot ? "The next time" : "If");
		}

		public bool IsParsed()
		{
			return Replaceable.IsParsed() && Ability.IsParsed();
		}

		public IEnumerable<UnknownPart> GetUnknownParts()
		{
			List<UnknownPart> unknowns = new List<UnknownPart>();
			unknowns.AddRange(Replaceable.GetUnknownParts());
			unknowns.AddRange(Ability.GetUnknownParts());
			return unknowns;
		}

		public bool IsProxy()
		{
			return Replaceable.IsProxy() || Ability.IsProxy();
		}

		public IAbility RemoveProxy()
		{
			Replaceable = Replaceable.RemoveProxy();
			Ability = Ability.RemoveProxy();
			return this;
		}

		public static readonly Parser<IAbility> Parser = new MyParser(CommonTokens.If, false);
		public static readonly Parser<IAbility> ParserNextTime = new MyParser(CommonTokens.TheNextTime, true);

		private class MyParser : Parser<IAbility>
		{
			private readonly string StartKw;
			private readonly bool IsOneShot;
			public MyParser(string startKw, bool oneShot)
			{
				StartKw = startKw;
				IsOneShot = oneShot;
			}

			public override bool Can(string text)
			{
				bool startsWithIf = text.StartsWith( StartKw, StringComparison.OrdinalIgnoreCase );
				if (!startsWithIf) return false;

				int iCs = text.IndexOf( CommonTokens.CommaSpace );
				int idxWould = text.IndexOf( CommonTokens.Would, StringComparison.OrdinalIgnoreCase );
				int idxInstead = text.IndexOf( CommonTokens.Instead, StringComparison.OrdinalIgnoreCase );
				int idxPrevent = text.IndexOf(CommonTokens.Prevent, StringComparison.OrdinalIgnoreCase);

				bool isWouldInstead = idxWould > 0 && idxWould < iCs && idxInstead > 0;
				bool isWouldPrevent = idxWould > 0 && idxWould < iCs && idxPrevent > 0;
				return isWouldPrevent || isWouldInstead;
			}

			protected override IAbility Parse( string text )
			{
				int iCs = text.IndexOf( CommonTokens.CommaSpace );
				string textTrigger = text.Substring(StartKw.Length, iCs - StartKw.Length);
				string abilityPart = text.Substring( iCs + CommonTokens.CommaSpace.Length ).Replace( CommonTokens.Instead, String.Empty ).Replace( "  ", " " ).Trim();
				ITrigger trigger = new UnparsedReplacementTrigger( textTrigger);
				return new ReplacementAbility { Ability = new UnparsedAbility(abilityPart), Replaceable = trigger, IsOneShot = IsOneShot };
			}
		}



		public AbContext SubParts
		{
			get { return Ability.SubParts; }
			set { Ability.SubParts = value; }
		}
	}
}
