﻿using System;
using System.Collections.Generic;
using System.Linq;
using AbilityParser.MagicPredicate;
using AbilityParser.Trigger;
using Magic.Utils;

namespace AbilityParser.Ability
{
	class TriggeredAbility : IAbility
	{
		internal ITrigger Trigger;
		internal IAbility Ability;
		internal string Kw;

		public override string ToString()
		{
			string condition = Trigger.ToString();
			string effect = Ability.ToString();
			return string.Format("{2}[{0}],\n  {{ {1} }}", condition, effect, Kw);
		}

		public bool IsParsed()
		{
			return Trigger.IsParsed() && Ability.IsParsed();
		}

		public IEnumerable<UnknownPart> GetUnknownParts()
		{
			List<UnknownPart> unknowns = new List<UnknownPart>();
			unknowns.AddRange(Trigger.GetUnknownParts());
			unknowns.AddRange(Ability.GetUnknownParts());
			return unknowns;
		}

		public bool IsProxy()
		{
			return Trigger.IsProxy() || Ability.IsProxy();
		}

		public IAbility RemoveProxy()
		{
			Trigger = Trigger.RemoveProxy();
			Ability = Ability.RemoveProxy();
			return this;
		}

		public static readonly Parser<IAbility> Parser = new TriggeredParser();
		private class TriggeredParser : Parser<IAbility>
		{
			public override bool Can(string text)
			{
				return CommonTokens.TriggerTokes.Any( t => text.StartsWith(t,StringComparison.OrdinalIgnoreCase) ) && text.IndexOf( CommonTokens.CommaSpace ) >= 0;
			}

			protected override IAbility Parse( string text )
			{
				string startsKw = CommonTokens.TriggerTokes.First( t => text.StartsWith( t, StringComparison.OrdinalIgnoreCase ) );

				int iCs = IndexOfCommaSpaceAfterSiblings( text );
				string textTrigger = text.Substring( startsKw.Length, iCs - startsKw.Length );
				string abilityPart = text.Substring( iCs + CommonTokens.CommaSpace.Length );
				ITrigger trigger = startsKw == CommonTokens.TriggerAt ? (ITrigger) new TriggerAt( textTrigger) : new UnparsedTrigger(textTrigger);
				return new TriggeredAbility {Ability = new UnparsedAbility( abilityPart ), Trigger = trigger };
			}
		}

		public AbContext SubParts
		{
			get { return Ability.SubParts; }
			set { Ability.SubParts = value; }
		}
	}
}
