﻿using System;
using System.Collections.Generic;
using System.Linq;
using Magic.Utils;

namespace AbilityParser.Ability
{
	public class UnparsedAbility : UnparsedUnit<IAbility>, IAbility
	{

		public UnparsedAbility( string abilityEffect ) : base( abilityEffect )
		{
			do {
				if (KeywordedStaticAbility.TryParse(Text, ref parsed)) break;
				if (KeywordedActivated.TryParse(Text, ref parsed)) break;
				if (TriggeredAbility.Parser.Try(Text, ref parsed)) break;
				if (AbilityAlternatives.Parser.Try(Text, ref parsed)) break;
				if (AbilitySequence.Parser.Try(Text, ref parsed)) break;
				if (ConditionedAbility.ParseUntil.Try(Text, ref parsed)) break;	// until your nex turn

				Text = discardThen( Text );

				if (ReplacementAbility.Parser.Try(Text, ref parsed)) break;
				if (ConditionedAbility.ParseIf.Try(Text, ref parsed)) break;
				if (ConditionedAbility.ParseAsLongAs.Try(Text, ref parsed)) break;
				if (ReplacementAbility.ParserNextTime.Try(Text, ref parsed)) break;

			} while( false );

			if (parsed != null && SubParts != null && !SubParts.IsEmpty() ) parsed.SubParts = this.SubParts;
		}

		private static string discardThen( string text )
		{
			if (text.StartsWith("Then ", StringComparison.OrdinalIgnoreCase)) return text.Substring( "Then ".Length );
			if (text.StartsWith("Then, ", StringComparison.OrdinalIgnoreCase)) return text.Substring("Then, ".Length);
			return text;
		}


		public override IEnumerable<UnknownPart> GetUnknownParts()
		{
			List<UnknownPart> result = new List<UnknownPart> {new UnknownPart( "Effect", Text, GetType() )};

			if (SubParts != null && SubParts.SubAbilities != null)
			{
				foreach (var c in SubParts.SubAbilities)
				{
					result.AddRange(c.GetUnknownParts());
				}
			}
			return result;
		}

		public override IAbility RemoveProxy()
		{
			return parsed != null ? parsed.RemoveProxy() : this;
		}

		public override string ToString()
		{
			string prefixKws = String.Empty;
			if ( SubParts != null && SubParts.UsefulKeyword != null )
			{
				prefixKws = String.Format( "(significant) {0} {1} ", SubParts.UsefulKeyword, CommonTokens.EmDash );
			}

			string subTexts = String.Empty;
			if ( SubParts != null && SubParts.SubAbilities != null )
			{
				subTexts = String.Join( String.Empty, SubParts.SubAbilities.Select( s => String.Format( "{0}  SubAbility: {1}", Environment.NewLine, s ) ) );
			}
			return String.Format("{0}*Unparsed* {1}{2}", prefixKws, Text, subTexts);
		}

		public AbContext SubParts { 
			get { return parsed == null ? _parts : parsed.SubParts; } 
			set { if ( parsed == null ) _parts = value; else parsed.SubParts = value; }
		}
		private AbContext _parts;

	}
}