﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AbilityParser.Ability;
using Magic.Utils;

namespace AbilityParser
{
	public class AbilityRefinerInitial : IAbility
	{

		protected IAbility effect;

		protected string uselessKW;

		public AbilityRefinerInitial(string rulesText, bool searchSubabilities = true)
		{
			SubParts = new AbContext();


			string text = rulesText.Trim(CommonTokens.TrimFromAbilities);
			text = searchSubabilities ? SplitOnQuotes(text) : SplitOnSecondLevelQuote( text );
			text = RemoveKeywordPrefixes( text );

			do
			{
				if( ActivatedAbility.Parser.Try( text, ref effect ) ) break;
				effect = new UnparsedAbility( text );
			} while( false );

			effect.SubParts = this.SubParts;
		}

		// removes useless prefixes, that are either written as text or
		protected string RemoveKeywordPrefixes( string source )
		{
			string result = source;
			uselessKW = CommonTokens.UselessKeywords.FirstOrDefault(result.StartsWith);
			if (null != uselessKW)
			{
				result = source.Substring(uselessKW.Length).Trim(CommonTokens.EmDashSpace);
			}
			SubParts.UsefulKeyword = CommonTokens.SignificantKeywords.FirstOrDefault(result.StartsWith);
			if (null != SubParts.UsefulKeyword)
			{
				result = source.Substring(SubParts.UsefulKeyword.Length).Trim(CommonTokens.EmDashSpace);
			}
			return result;
		}

		// Only Koth of the Hammer uses this feature
		protected string SplitOnSecondLevelQuote(string source)
		{
			int idxQ2 = source.IndexOf( CommonTokens.QuoteL2 );
			if (-1 == idxQ2) return source;
			int idxApos = source.IndexOf( CommonTokens.QuoteL2Close );

			string prolog = source.Substring( 0, idxQ2 );
			StringBuilder sb = new StringBuilder();

			sb.Append( prolog );
			sb.Append( "[SubAbility0]" );
			sb.Append( source.Substring( idxApos + 1 ) );

			SubParts.SubAbilities = new List<IAbility>(1);
			string lvl2Text = source.Substring(idxQ2 + 1, idxApos - (idxQ2 + 1));
			SubParts.SubAbilities.Add(new AbilityRefinerInitial(lvl2Text, false));
			return sb.ToString();
		}


		protected string SplitOnQuotes(string source)
		{
			string[] parts = source.Split(CommonTokens.Quotes);
			if(parts.Length == 0) return parts[0];

			SubParts.SubAbilities = new List<IAbility>(parts.Length / 2);
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < parts.Length; i++)
			{
				if (i % 2 == 0)
				{
					sb.Append(parts[i]);
				}
				else
				{
					sb.AppendFormat("[SubAbility{0}]", i / 2);
					SubParts.SubAbilities.Add(new AbilityRefinerInitial(parts[i], false));
				}
			}
			return sb.ToString();
		}


		public bool IsParsed()
		{
			return false;
		}

		public IEnumerable<UnknownPart> GetUnknownParts()
		{
			List<UnknownPart> result = new List<UnknownPart>();
			result.AddRange( effect.GetUnknownParts() );

			if (SubParts.SubAbilities != null)
			{
				foreach (var c in SubParts.SubAbilities)
				{
					result.AddRange( c.GetUnknownParts() );
				}
			}
			return result;
		}

		public bool IsProxy()
		{
			return true;
		}

		public IAbility RemoveProxy()
		{
			for (int i = 0; SubParts.SubAbilities != null && i < SubParts.SubAbilities.Count; i++)
			{
				SubParts.SubAbilities[i] = SubParts.SubAbilities[i].RemoveProxy();
			}
			return effect.RemoveProxy();
		}

		public override string ToString()
		{
			string usefulprefix = SubParts.UsefulKeyword != null ? String.Format( "(significant) {0} {1} ", SubParts.UsefulKeyword, CommonTokens.EmDash ) : String.Empty;
			string anyPrefixKws = uselessKW != null ? String.Format("(useless) {0} {1} ", uselessKW, CommonTokens.EmDash) : usefulprefix;
			string subTexts = SubParts.SubAbilities == null ? String.Empty : String.Join(String.Empty, SubParts.SubAbilities.Select(s => String.Format("{0}  SubAbility: {1}", Environment.NewLine, s)));
			return String.Format( "{0}*Unparsed* {1}{2}", anyPrefixKws, effect, subTexts );
		}

		public AbContext SubParts { get; set; }
	}
}