﻿using System;
using System.Collections.Generic;
using AbilityParser.Ability;
using AbilityParser.Calculator;
using AbilityParser.Cost;
using AbilityParser.Effect;
using AbilityParser.Filter;
using AbilityParser.Rules;
using Magic.Mana;
using Magic.Utils;


namespace AbilityParser
{
	class BasicLandParser
	{
		public static Dictionary<String,byte[]> hardCodedManasources = new Dictionary<string, byte[]>
		{
			//                                W  U  B  R  G 
			{ "Plains",           new byte[] {1, 0, 0, 0, 0} },
			{ "Island",           new byte[] {0, 1, 0, 0, 0} },
			{ "Swamp",            new byte[] {0, 0, 1, 0, 0} },
			{ "Mountain",         new byte[] {0, 0, 0, 1, 0} },
			{ "Forest",           new byte[] {0, 0, 0, 0, 1} },

			//                                      W  U  B  R  G 
			{ "Snow-Covered Plains",    new byte[] {1, 0, 0, 0, 0} },
			{ "Snow-Covered Island",    new byte[] {0, 1, 0, 0, 0} },
			{ "Snow-Covered Swamp",     new byte[] {0, 0, 1, 0, 0} },
			{ "Snow-Covered Mountain",  new byte[] {0, 0, 0, 1, 0} },
			{ "Snow-Covered Forest",    new byte[] {0, 0, 0, 0, 1} },

			//                                W  U  B  R  G 
			{ "Badlands",         new byte[] {0, 0, 1, 1, 0} },
			{ "Bayou",            new byte[] {0, 0, 1, 0, 1} },
			{ "Plateau",          new byte[] {1, 0, 0, 1, 0} },
			{ "Savannah",         new byte[] {1, 0, 0, 0, 1} },
			{ "Scrubland",        new byte[] {1, 0, 1, 0, 0} },
			{ "Taiga",            new byte[] {0, 0, 0, 1, 1} },
			{ "Tropical Island",  new byte[] {0, 1, 0, 0, 1} },
			{ "Tundra",           new byte[] {1, 1, 0, 0, 0} },
			{ "Underground Sea",  new byte[] {0, 1, 1, 0, 0} },
			{ "Volcanic Island",  new byte[] {0, 1, 0, 1, 0} },
		};

		public static bool TryParse(String cardName, String text, IList<IAbility> result)
		{
			byte[] profile;
			if (!hardCodedManasources.TryGetValue(cardName, out profile) )
				return false;

			ManaColor[] colors = new[] { ManaColor.White, ManaColor.Blue, ManaColor.Black, ManaColor.Red, ManaColor.Green };

			for(int i = 0; i < colors.Length; i++) {
				if( profile[i] > 0 )
					result.Add( getManaForTapSelf( colors[i] ) );
			}
			return true;
		}

		private static IAbility getManaForTapSelf(ManaColor color)
		{
			return new ActivatedAbility
			{
				Costs = new CostEffect(new ChangeCardState(CardBinaryState.Tapped, new FilterAbilitySource())), 
				Ability = new OneShotEffect( new GiveMana { Amount = new CalculatorConstant<int>(1), Color = new CalculatorConstant<ManaColor>(color) } )
			};
		}

	}
}
