﻿using System;
using System.Collections.Generic;
using AbilityParser.Effect;
using AbilityParser.Filter;
using AbilityParser.Rules;
using Magic.Utils;

namespace AbilityParser.Cost
{
	class UnparsedCost : ICost
	{
		private readonly string Text;
		private ICost parsed = null;

		public UnparsedCost(string text)
		{
			Text = text;
			if (CostComposite.ParseDisjunction.Try(text, ref parsed)) return;
			if( CostComposite.ParseConjunction.Try( text, ref parsed ) ) return;

			if( text == "{T}" )
			{
				parsed = new CostEffect( new ChangeCardState( CardBinaryState.Tapped, new FilterAbilitySource() ) );
				return;
			}
			if (text == "{Q}")
			{
				parsed = new CostEffect(new ChangeCardState(CardBinaryState.Tapped, new FilterAbilitySource(), false));
				return;
			}

			if (CostMana.Parser.Try(text, ref parsed)) return;
			if (CostLoyalty.Parser.Try(text, ref parsed)) return;
		}

		public bool IsParsed()
		{
			return false;
		}

		public IEnumerable<UnknownPart> GetUnknownParts()
		{
			yield return new UnknownPart("Cost", Text, GetType());
		}

		public bool IsProxy()
		{
			return true;
		}

		public ICost RemoveProxy()
		{
			return parsed != null ? parsed.RemoveProxy() : this;
		}

		public override string ToString()
		{
			return "*Unparsed Cost* " + Text;
		}
	}
}
