﻿using System;
using System.Collections.Generic;
using System.Linq;
using AbilityParser.Ability;
using AbilityParser.Condition;
using AbilityParser.Cost;
using AbilityParser.Entities;
using AbilityParser.Filter;
using AbilityParser.MagicPredicate;
using Magic.Cards;
using Magic.Utils;


namespace AbilityParser
{
	public class RootParser
	{
		public static void Parse(CardRulesInner card)
		{
			int nRules = card.Texts.Count;
			List<IAbility> result = new List<IAbility>();
			string cardName = card.Name;
			bool[] parsed = new bool[nRules];
			
			if ( BasicLandParser.TryParse( cardName, null, result ) )
			{
				for( int i = 0; i < parsed.Length; i++ ) { parsed[i] = true; }
				card.setAbilities( result, parsed );
				return;
			}

			if (nRules == 0) { return; }

			// Process levellers here (1 line for levelup cost, 3 lines for their stats)
			string firstRule = card.Texts[0];
			bool isLeveller = false;

			if (firstRule.StartsWith(CommonTokens.LevelUp))
			{
				int idxCost = ActivatedAbility.ActivatedAbilityParser.IndexOfCostNoParams( CommonTokens.LevelUp, firstRule );
				if (idxCost >= 0)
				{
					isLeveller = true;
					string manaCost = firstRule.Substring( idxCost );
					var ability = new ActivatedAbility {Ability = new KeywordedActivated( CommonTokens.LevelUp ), Costs = new UnparsedCost( manaCost )};
					result.Add( ability.RemoveProxy() );
				}
			}

			if (!isLeveller) // common creatures (abilities not linked that much)
			{
				string[] rules = card.Texts.Select( ruleText => ReplaceCardName( cardName, ruleText ) ).ToArray();

				// Had to apply errata here.
				if (cardName == "Planeswalker's Mischief")
				{
					rules[0] = rules[0].Replace( "(If it has X in its mana cost, X is 0.) ", string.Empty );
				}
				var abilities = rules.Select( FetchAbilityAnyway );
				result.AddRange( abilities.Select( unit => unit.RemoveProxy() ));
			}
			else // levellers' ability lines are senseless without context
			{
				const string levelToken = "LEVEL ";
				int lastLevelLine = -1;
				ICondition levelCondition = null;
				for (int iRule = 1; iRule < card.Texts.Count; iRule++)
				{
					string ruleText = card.Texts[iRule];
					if (ruleText.StartsWith(levelToken))
					{
						string levels = ruleText.Substring( levelToken.Length );
						string[] levelRange = levels.Split( CommonTokens.PlusMinus, StringSplitOptions.RemoveEmptyEntries );
						//System.Diagnostics.Debug.WriteLine( cardName + levels );
						int minLevel = int.Parse( levelRange[0] );
						int maxLevel = levelRange.Length > 1 ? int.Parse( levelRange[1] ) : int.MaxValue;
						levelCondition = new FilteredCondition<Card>
						        	{Filter = new FilterAbilitySource(), MagicPredicate = new CounterPredicate {CounterName = "LEVEL", MaxAmount = maxLevel, MinAmount = minLevel}};
						lastLevelLine = iRule;
						continue;
					}

					if (lastLevelLine + 1 == iRule)
					{
						// TODO: should avoid confusing - presence of counters does not overwrite card type to creature
						string abilityText = String.Format( "{0} is a {1} creature", CommonTokens.ThisCardToken, ruleText );
						var toAdd = new ConditionedAbility {Condition = levelCondition, Ability = new AbilityRefinerInitial( abilityText, false )};
						result.Add( toAdd.RemoveProxy() );
						continue;
					}

					IAbility unit = FetchAbilityAnyway(ReplaceCardName(cardName, ruleText));
					unit = new ConditionedAbility {Ability = unit, Condition = levelCondition};
					result.Add(unit.RemoveProxy());
				}
			}

			card.setAbilities( result, parsed );
		}

		private static IAbility FetchAbilityAnyway(string text)
		{
			return new AbilityRefinerInitial(text);
		}

		// elimitaes double spaces too.
		private static string ReplaceCardName( string cardName, string ruleText )
		{
			string result = ruleText.Replace( cardName, CommonTokens.ThisCardToken );
			int commaIndex = cardName.IndexOf( ", " );
			if ( commaIndex >= 0 )
			{
				string shortName = cardName.Substring( 0, commaIndex );
				result = result.Replace(shortName, CommonTokens.ThisCardToken);
			}
			return result.Replace("  ", " ").Replace( ".\"", "\"." );
		}
	}
}
