﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

namespace Magic.Mana
{
	public class ManaCost : IComparable<ManaCost>
	{
		public ReadOnlyCollection<ManaCostShard> Shards { get { return shards; } }
		private readonly ReadOnlyCollection<ManaCostShard> shards;
		public int Generic { get { return generic; } }
		private readonly int generic;
		public string stringValue; // precalculated for toString;
		
		private readonly float compareWeight;
		private static ManaCost noCost = new ManaCost( null, 0 );

		private ManaCost( ReadOnlyCollection<ManaCostShard> readOnlyCollection, int genericCost )
		{
			shards = readOnlyCollection;
			generic = genericCost;
			compareWeight = shards == null ? -1 : generic + shards.Sum(s => s.CmpCost);
			stringValue = SimpleString;
		}

		public static ManaCost Parse( string value )
		{
			if (String.IsNullOrWhiteSpace( value )) return noCost;
			IList<ManaCostShard> shardsTemp = new List<ManaCostShard>();

			char[] line = value.Trim().ToCharArray();
			int len = line.Length;
			int idx = 0;
			int genericCost = 0;
			while (idx >= 0 && idx < len)
			{
				ManaCostShard shard = extractShard( line, ref idx, ref genericCost );
				if ( null != shard ) shardsTemp.Add(shard); // null is OK - that was generic mana
			}
			return new ManaCost( new ReadOnlyCollection<ManaCostShard>( shardsTemp ), genericCost );
		}

		 public byte ColorProfile {get {
			byte result = 0;
		 	if ( shards != null ) for( int i = 0; i < shards.Count; i++ )
		 	{
		 		ManaCostShard s = shards[i];
		 		result |= s.ColorMask;
		 	}
		 	return result;
		}}

		private static ManaCostShard extractShard(char[] value, ref int idx, ref int generic)
		{
			char here = value[idx++];
			if (Char.IsDigit(here))
			{
				int toAdd = int.Parse( here.ToString() );
				if (idx < value.Length && Char.IsDigit(value[idx]))
				{
					toAdd = 10*toAdd + int.Parse( value[idx++].ToString() );
				}
				generic += toAdd;
				return null;
			}
			
			int manaColors;
			if( here == '{' && value[idx+1] == '/')
			{
				manaColors = toManaAtom( value[idx] ) | toManaAtom( value[idx + 2] );
				idx += 4;
			}
			else if (here == '{' )
			{
				here = value[idx++];
				if (Char.IsDigit(here))
				{
					int toAdd = int.Parse(here.ToString());
					if (idx < value.Length && Char.IsDigit(value[idx]))
					{
						toAdd = 10 * toAdd + int.Parse(value[idx++].ToString());
					}
					generic += toAdd;
					idx++; // skip }
					return null;
				}
				manaColors = toManaAtom( here );
				idx += 1;
			}

			else 
				manaColors = toManaAtom( here );


			return ManaCostShard.ValueOf( manaColors );
		}

		private static short toManaAtom(char symbol)
		{
			switch( symbol )
			{
				case 'W':
					return ManaCostAtom.White;
				case 'U':
					return ManaCostAtom.Blue;
				case 'B':
					return ManaCostAtom.Black;
				case 'R':
					return ManaCostAtom.Red;
				case 'G':
					return ManaCostAtom.Green;
				case '2':
					return ManaCostAtom.Or2Mana;
				case 'P':
					return ManaCostAtom.Or2Life;
				case 'X':
					return ManaCostAtom.IsX;
				case 'S':
					return ManaCostAtom.Snow;
			}
			return 0;
		}

		public int CompareTo( ManaCost other )
		{
			if( other == null ) return 1;
			float delta = compareWeight - other.compareWeight;
			if (delta == 0) return 0;
			return delta > 0 ? 1 : -1;
		}

		private String SimpleString { get {
			if(shards == null) return "-";
			if (!shards.Any()) { return generic.ToString(); }

			StringBuilder sb = new StringBuilder();
			bool isFirst = true;
			if (generic > 0) { sb.Append(generic); isFirst = false; }
			foreach (ManaCostShard s in shards) {
					if (!isFirst) { sb.Append(' '); }
					else { isFirst = false; }
					sb.Append(s.ToString());
			}
			return sb.ToString();
		}}

		public override string ToString() { return stringValue; }
	}
}
