﻿using System;
using System.Linq;
using Magic.Common;

namespace Magic.Mana
{
	public sealed class ManaCostShard
	{
		public static readonly ManaCostShard X = new ManaCostShard(ManaCostAtom.IsX, "X");
		public static readonly ManaCostShard S = new ManaCostShard(ManaCostAtom.Snow, "S");

		public readonly static ManaCostShard White = new ManaCostShard(ManaCostAtom.White, "W");
		public readonly static ManaCostShard Blue = new ManaCostShard(ManaCostAtom.Blue, "U");
		public readonly static ManaCostShard Black = new ManaCostShard(ManaCostAtom.Black, "B");
		public readonly static ManaCostShard Red = new ManaCostShard(ManaCostAtom.Red, "R");
		public readonly static ManaCostShard Green = new ManaCostShard(ManaCostAtom.Green, "G");

		public readonly static ManaCostShard PW = new ManaCostShard(ManaCostAtom.White | ManaCostAtom.Or2Life, "W/P");
		public readonly static ManaCostShard PU = new ManaCostShard(ManaCostAtom.Blue | ManaCostAtom.Or2Life, "U/P");
		public readonly static ManaCostShard PB = new ManaCostShard(ManaCostAtom.Black | ManaCostAtom.Or2Life, "B/P");
		public readonly static ManaCostShard PR = new ManaCostShard(ManaCostAtom.Red | ManaCostAtom.Or2Life, "R/P");
		public readonly static ManaCostShard PG = new ManaCostShard(ManaCostAtom.Green | ManaCostAtom.Or2Life, "G/P");

		public readonly static ManaCostShard WU = new ManaCostShard(ManaCostAtom.White | ManaCostAtom.Blue, "W/U");
		public readonly static ManaCostShard WB = new ManaCostShard(ManaCostAtom.White | ManaCostAtom.Black, "W/B");
		public readonly static ManaCostShard WR = new ManaCostShard(ManaCostAtom.White | ManaCostAtom.Red, "W/R");
		public readonly static ManaCostShard WG = new ManaCostShard(ManaCostAtom.White | ManaCostAtom.Green, "W/G");
		public readonly static ManaCostShard UB = new ManaCostShard(ManaCostAtom.Blue | ManaCostAtom.Black, "U/B");
		public readonly static ManaCostShard UR = new ManaCostShard(ManaCostAtom.Blue | ManaCostAtom.Red, "U/R");
		public readonly static ManaCostShard UG = new ManaCostShard(ManaCostAtom.Blue | ManaCostAtom.Green, "U/G");
		public readonly static ManaCostShard BR = new ManaCostShard(ManaCostAtom.Black | ManaCostAtom.Red, "B/R");
		public readonly static ManaCostShard BG = new ManaCostShard(ManaCostAtom.Black | ManaCostAtom.Green, "B/G");
		public readonly static ManaCostShard RG = new ManaCostShard(ManaCostAtom.Red | ManaCostAtom.Green, "R/G");

		public readonly static ManaCostShard W2 = new ManaCostShard(ManaCostAtom.White | ManaCostAtom.Or2Mana, "2/W");
		public readonly static ManaCostShard U2 = new ManaCostShard(ManaCostAtom.Blue | ManaCostAtom.Or2Mana, "2/U");
		public readonly static ManaCostShard B2 = new ManaCostShard(ManaCostAtom.Black | ManaCostAtom.Or2Mana, "2/B");
		public readonly static ManaCostShard R2 = new ManaCostShard(ManaCostAtom.Red | ManaCostAtom.Or2Mana, "2/R");
		public readonly static ManaCostShard G2 = new ManaCostShard(ManaCostAtom.Green | ManaCostAtom.Or2Mana, "2/G");
		
		private static readonly ManaCostShard[] allPossible = new [] {
			X, S, White, Blue, Black, Red, Green,
			PW, PU, PB, PR, PG, W2, U2, B2, R2, G2,
			WU, WB, WR, WG, UB, UR, UG, BR, BG, RG
		};

		private readonly short shard;
		private readonly string stringValue;
		public byte Cmc { get { return 0 == ( shard & ManaCostAtom.IsX ) ? ( 0 == ( shard & ManaCostAtom.Or2Mana ) ? (byte) 1 : (byte) 2 ) : (byte) 0; } }
		private ManaCostShard(short shard, string value)
		{
			this.shard = shard;
			stringValue = value;
		}

		public override string ToString()
		{
			return stringValue;
		}

		/**
		 * Returns Mana cost, adjusted slightly to make colored mana parts more significant.
		 * Should only be used for comparison purposes; using this method allows the sort:
		 * 2 < X 2 < 1 U < U U < UR U < X U U < X X U U
		 *
		 * @return The converted cost + 0.0005* the number of colored mana in the cost + 0.00001 *
		 *         the number of X's in the cost
		 */
		public float CmpCost { get {
			if (0 != (shard & ManaCostAtom.IsX)) { return 0.0001f; }
			float cost = 0 != (shard & ManaCostAtom.Or2Mana) ? 2 : 1;
			// yes, these numbers are magic, slightly-magic
			if (0 != (shard & ManaCostAtom.White)) { cost += 0.0005f; }
			if (0 != (shard & ManaCostAtom.Blue)) { cost += 0.0020f;  }
			if (0 != (shard & ManaCostAtom.Black)) { cost += 0.0080f;  }
			if (0 != (shard & ManaCostAtom.Red)) { cost += 0.0320f; }
			if (0 != (shard & ManaCostAtom.Green)) { cost += 0.1280f; }
			if (0 != (shard & ManaCostAtom.Or2Life)) { cost += 0.00003f; }
			return cost;
		}}

		public byte ColorMask { get {
			byte result = 0;
			if (0 != (shard & ManaCostAtom.White)) { result |= MagicColor.White; }
			if (0 != (shard & ManaCostAtom.Blue)) { result |= MagicColor.Blue; }
			if (0 != (shard & ManaCostAtom.Black)) { result |= MagicColor.Black; }
			if (0 != (shard & ManaCostAtom.Red)) { result |= MagicColor.Red; }
			if (0 != (shard & ManaCostAtom.Green)) { result |= MagicColor.Green; }
			return result;
		}}

		public static ManaCostShard ValueOf(int atoms)
		{
			try
			{
				return allPossible.First(t => t.shard == atoms);
			}
			catch (Exception ex)
			{
				System.Diagnostics.Debug.Print( atoms.ToString("X") );
				throw;
			}
		}
	}
}
